import './common.css'
import './minor.css'
import './content.css'
import './App.css'
import { format } from 'date-fns';
import { GalleryContent } from './Gallery';
import { GalleryTitleHeader } from './GalleryTitleHeader';


function getTimeFromNow(date: Date) {
  const now = new Date()
  if (now.getFullYear() !== date.getFullYear()) {
    return format(date, "yyyy.MM.dd")
  }
  return format(date, "HH:mm")
}

type Writer = {
  type: "유동닉",
  /**
   * IP address without the last two octet
   * e.g. "192.168"
   */
  ip: string
} | {
  type: "반유동"
} | {
  type: "고닉",
  nickname: string,
  완장?: "주딱" | "파딱"
} | {
  type: "운영자"
};

interface GalleryTableRowProps {
  id: number
  subject: string
  title: string
  replyCount: number
  writer: Writer
  type: "icon_notice" | "icon_pic" | "icon_txt" | "icon_survey"
  date: Date
  viewCount: number
  recommendCount: number
  fixed?: boolean
}

const NicknameImagePath = {
  "주딱": "/fix_managernik.gif",
  "파딱": "/fix_sub_managernik.gif",
  "반유동": "/nik.gif",
  "default": "/fix_nik.gif"
}
function RenderWriter({ writer }: { writer: Writer }) {
  const type = writer.type;
  if (type === "운영자") {
    return <span className="nickname in" title="운영자"><em>운영자</em></span>
  }
  const nickname = writer.type === "고닉" ? writer.nickname : "ㅇㅇ";
  return <>
    <span className="nickname in" title={nickname}><em>{nickname}</em></span>
    {writer.type === "유동닉" && <span className="ip">({writer.ip})</span>}
    <span className="writer_nikcon">
      {writer.type === "고닉" && <img src={NicknameImagePath[writer.완장 ?? "default"]} title={`${nickname} : 완장`}
        width="12" height="11" style={{ "cursor": "pointer", marginLeft: "2px" }}
        alt="갤로그로 이동합니다." />}
      {writer.type === "반유동" && <img src={NicknameImagePath["반유동"]} title={`${nickname} : 닉네임`}
        width="12" height="11" style={{ "cursor": "pointer", marginLeft: "2px" }}
        alt="갤로그로 이동합니다." />}
    </span>
  </>
}

function GalleryTableRow(
  {
    id,
    subject,
    title,
    replyCount,
    writer,
    type,
    date,
    viewCount,
    recommendCount,
    fixed = false,
  }: GalleryTableRowProps
) {
  const wrap = (node: React.ReactNode) => fixed ? <b>{node}</b> : node;

  return <tr className="ub-content us-post" data-no={id.toString()} data-type={type}>
    <td className="gall_num">{id}</td>
    <td className="gall_subject"><b>{subject}</b></td>
    <td className="gall_tit ub-word">
      <a href="/mgallery/board/view/?id=genrenovel&amp;no=9101266&amp;page=1" view-msg="">
        <em className={`icon_img ${type}`}></em>
        {wrap(title)}
      </a>
      <a className="reply_numbox">
        <span className="reply_num">[{replyCount}]</span>
      </a>
    </td>
    <td className="gall_writer ub-writer" data-nick={writer} data-ip="" data-loc="list">
      {wrap(
        <RenderWriter writer={writer} />
      )}
    </td>
    <td className="gall_date" title={format(date, "yyyy-MM-dd hh:mm:ss")}>
      {getTimeFromNow(date)}
    </td>
    <td className="gall_count">{viewCount}</td>
    <td className="gall_recommend">{recommendCount}</td>
  </tr>
}

function GalleryTable() {
  return <div className='gall_listwrap list'>
    <table className="gall_list">
      <caption>갤러리 리스트</caption>
      <colgroup>
        <col style={{ "width": "7%" }} />
        <col style={{ "width": "51px" }} />
        <col />
        <col style={{ "width": "18%" }} />
        <col style={{ "width": "6%" }} />
        <col style={{ "width": "6%" }} />
        <col style={{ "width": "6%" }} />
      </colgroup>
      <thead>
        <tr>
          <th scope="col">번호</th>
          <th scope="col">말머리</th>
          <th scope="col">제목</th>
          <th scope="col">글쓴이</th>
          <th scope="col">작성일</th>
          <th scope="col">조회</th>
          <th scope="col">추천</th>
        </tr>
      </thead>
      <tbody className="listwrap2 ">

        <GalleryTableRow
          id={2973}
          subject="설문"
          title="입금 전,후 관리에 따라 외모 갭이 큰 스타는?"
          writer={{ type: "운영자" }}
          type="icon_survey"
          date={new Date("2025-01-20 22:52:52")}
          viewCount={0}
          recommendCount={0}
          replyCount={0}
        />

        <GalleryTableRow
          id={9101266}
          subject="공지"
          title="장르소설 마이너 갤러리 공지 모음"
          writer={{ type: "고닉", nickname: "apocalypse", 완장: "파딱" }}
          type="icon_notice"
          date={new Date("2024-08-08 23:55:43")}
          viewCount={35080}
          recommendCount={21}
          replyCount={18}
        />

        <GalleryTableRow
          id={9073478}
          subject="공지"
          title="주딱 문의글"
          writer={{ type: "유동닉", ip: "223.39" }}
          type="icon_notice"
          date={new Date("2024-08-04 07:24:58")}
          viewCount={45439}
          recommendCount={24}
          replyCount={28}
        />

        <GalleryTableRow
          id={9056755}
          subject="공지"
          title="신문고"
          writer={{ type: "유동닉", ip: "118.235" }}
          type="icon_notice"
          date={new Date("2024-08-01 06:47:52")}
          viewCount={141598}
          recommendCount={45}
          replyCount={11}
        />

        <GalleryTableRow
          id={9861519}
          subject="리뷰"
          title="[리뷰대회]이민갔는데 딸이 생겼다"
          writer={{ type: "고닉", nickname: "나이브르" }}
          type="icon_pic"
          date={new Date("2025-01-20 22:52:52")}
          viewCount={10}
          recommendCount={1}
          replyCount={2}
        />

        <GalleryTableRow
          id={9861518}
          subject="일반"
          title="사펑불퇴 비처녀 음해는 그냥 글을 안읽은거 아니냐?"
          writer={{ type: "유동닉", ip: "118.45" }}
          type="icon_txt"
          date={new Date("2025-01-20 22:52:51")}
          viewCount={9}
          recommendCount={0}
          replyCount={0}
        />

        <GalleryTableRow
          id={9861517}
          subject="일반"
          title="역변영애 칼릭스를 진 남주로 할거였으면"
          writer={{ type: "유동닉", ip: "220.89" }}
          type="icon_txt"
          date={new Date("2025-01-20 22:52:42")}
          viewCount={5}
          recommendCount={0}
          replyCount={0}
        />

        <GalleryTableRow
          id={9861516}
          subject="일반"
          title="미3술가 좀 식네"
          writer={{ type: "고닉", nickname: "오이탕후루" }}
          type="icon_pic"
          date={new Date("2025-01-20 22:52:31")}
          viewCount={7}
          recommendCount={0}
          replyCount={1}
        />
      </tbody>
    </table>
  </div>
}

function App() {
  return (
    <>
      <div
        style={{
          width: "1160px",
          margin: "20px auto 0",
        }}
      >
        <GalleryTitleHeader />
        <div
          style={{
            borderTop: "2px solid #29367c",
            width: "1158px"
          }}
        />
        <GalleryContent />
      </div>
      <GalleryTable />
    </>
  )
}

export default App
